# ifndef FORMULA_H
# define FORMULA_H

# include <vector>

# include "../map/assignment.h"


/**
 *
 * This class represents a general formula. Its structured as a tree
 */
class Formula {
	public:
		Formula();
		virtual ~Formula();
		
		/*
		 * Returns the number of elements of this formula.
		 */
		virtual int getNumberOfElements();

		/*
		 * @param index: the index of the element to be retrieved.
		 * Returns the element at @index of this formula, if possible.
		 * Returns NULL otherwise.
		 */
		virtual Formula* getElement (int index);

		/*
		 * Returns the IDValue of this formula, if possible.
		 * Returns -1 otherwise.
		 */
		virtual int getIDValue ();

		/*
		 * @param f: the formula to be compared.
		 * Returns true if this instance equals f. Returns false otherwise.
		 */
		virtual bool equals (Formula *f);

		/*
		 * @param v: the assignment to be applied.
		 * Returns the value of this formula, when v is applied.
		 * Returns false if no formula is pointed.
		 */
		virtual bool getValue (Assignment *v);

		/*
		 * prints this formula in the standard output.
		 */
		virtual void print();

		/*
		 * Clear and delete recursively all subformulas. This instance isn't
		 * deleted.
		 */
		virtual void clear();

		/*
		 * erases recursively this formula and all subformulas.
		 */
		virtual void erase(); };


/**
 *
 * This class represents a constant TRUE formula
 *   It translate into boolean algebra as '1'
 */
class FormulaTrue: public Formula {
	public:
		FormulaTrue ();
		~FormulaTrue ();

		bool equals (Formula *f);
		bool getValue (Assignment *v);
		void print(); };


/**
 *
 * This class represents a constant FALSE formula
 *   It translate into boolean algebra as '0'
 */
class FormulaFalse: public Formula {
	public:
		FormulaFalse ();
		~FormulaFalse ();

		bool equals (Formula *f);
		bool getValue (Assignment *v);
		void print(); };


/**
 *
 * This class represents a formula specified by an identifier (represented by
 * its ID)
 *   It translate into boolean algebra by the value of the associated variable
 *   (note that this association is kept in the parser who generated the ID)
 */
class FormulaIdentif: public Formula {
	public:
		FormulaIdentif (int IDValue);
		~FormulaIdentif ();

		int getIDValue ();
		bool equals (Formula *f);
		bool getValue (Assignment *v);
		void print();

	private:
		int IDValue; };


/**
 *
 * This class represents a box operator formula
 */
class FormulaBox: public Formula {
	public:
		FormulaBox (Formula *element);
		~FormulaBox ();

		int getNumberOfElements();
		Formula* getElement (int index);
		bool equals (Formula *f);
		bool getValue (Assignment *v);
		void print();
		void clear();

	private:
		Formula *element; };


/**
 *
 * This class represents a formula with an AND operation as the root
 *   It translate into boolean algebra as multiplication
 */
class FormulaAnd: public Formula {
	public:
		FormulaAnd (std::vector<Formula*> *elements);
		~FormulaAnd ();

		int getNumberOfElements();
		Formula* getElement (int index);
		bool equals (Formula *f);
		bool getValue (Assignment *v);
		void print();
		void clear();

	private:
        std::vector<Formula*> *elements; };


/**
 *
 * This class represents a formula with an XOR operation as the root
 *   It translate into boolean algebra as sum
 */
class FormulaXor: public Formula {
	public:
		FormulaXor (std::vector<Formula*> *elements);
		~FormulaXor ();

		int getNumberOfElements();
		Formula* getElement (int index);
		bool equals (Formula *f);
		bool getValue (Assignment *v);
		void print();
		void clear();

	private:
        std::vector<Formula*> *elements; };


/**
 *
 * This class represents a whole sequent
 */
class Sequent: public Formula {
	public:
		Sequent (std::vector<Formula*> *premises, Formula *conclusion);
		~Sequent ();

		bool equals (Formula *f);
		bool getValue (Assignment *v);
		void print();
		void clear();

	private:
        std::vector<Formula*> *premises;
		Formula *conclusion; };



# endif // FROMULA_H
